unit uTestSqlite;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls,SQLiteTable3,ExtCtrls, jpeg;

type
  TForm1 = class(TForm)
    btnTest: TButton;
    memNotes: TMemo;
    Label1: TLabel;
    Label2: TLabel;
    ebName: TEdit;
    Label3: TLabel;
    ebNumber: TEdit;
    Label4: TLabel;
    ebID: TEdit;
    Image1: TImage;
    btnLoadImage: TButton;
    btnBackup: TButton;
    btnBack: TButton;
    btnForward: TButton;
    procedure btnTestClick(Sender: TObject);
    procedure btnLoadImageClick(Sender: TObject);
    procedure btnBackupClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure updateFields;
    procedure updateImage;
    procedure btnBackClick(Sender: TObject);
    procedure btnForwardClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    sltb: TSQLIteTable;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.btnTestClick(Sender: TObject);
var
slDBpath: string;
sldb: TSQLiteDatabase;
sSQL: String;
ts: TStringStream;

begin

slDBPath := ExtractFilepath(application.exename)
+ 'test.db';

sldb := TSQLiteDatabase.Create(slDBPath);
try

if sldb.TableExists('testTable') then begin
sSQL := 'DROP TABLE testtable';
sldb.execsql(sSQL);
end;

sSQL := 'CREATE TABLE testtable ([ID] INTEGER PRIMARY KEY,[OtherID] INTEGER NULL,';
sSQL := sSQL + '[Name] VARCHAR (255),[Number] FLOAT, [notes] BLOB, [picture] BLOB COLLATE NOCASE);';

sldb.execsql(sSQL);

sldb.execsql('CREATE INDEX TestTableName ON [testtable]([Name]);');

//begin a transaction
sldb.BeginTransaction;

sSQL := 'INSERT INTO testtable(Name,OtherID,Number) VALUES ("Some Name",4,587.6594);';
//do the insert
sldb.ExecSQL(sSQL);


sSQL := 'INSERT INTO testtable(Name,OtherID,Number,Notes) VALUES ("Another Name",12,4758.3265,"More notes");';
//do the insert
sldb.ExecSQL(sSQL);

//end the transaction
sldb.Commit;

//add the notes using a parameter
ts := TStringStream.Create('Here are some notes with a unicode smiley: ' + char($263a),TEncoding.UTF8);
try

//insert the text into the db
sldb.UpdateBlob('UPDATE testtable set notes = ? WHERE OtherID = 4',ts);

finally
ts.Free;
end;

if sltb<> nil then
sltb.Free;

//query the data
sltb := slDb.GetTable('SELECT * FROM testtable');

if sltb.Count > 0 then
begin
//display first row

updateFields;

end;

finally
sldb.Free;

end;

end;

procedure TForm1.updateFields;
var
Notes: string;

begin
ebName.Text := sltb.FieldAsString(sltb.FieldIndex['Name']);
ebID.Text := inttostr(sltb.FieldAsInteger(sltb.FieldIndex['ID']));
ebNumber.Text := floattostr( sltb.FieldAsDouble(sltb.FieldIndex['Number']));
Notes :=  sltb.FieldAsBlobText(sltb.FieldIndex['Notes']);
memNotes.Text := notes;
updateImage;
end;


procedure TForm1.UpdateImage;
var
ms: TMemoryStream;
pic: TJPegImage;
sldb: TSqliteDatabase;
sltbU: TSqliteUniTable;
slDBPath: string;
iID: integer;
begin

if sltb = nil then
exit;

self.Image1.Picture.Graphic := nil;

slDBPath := ExtractFilepath(application.exename)
+ 'test.db';

if not FileExists(slDBPath) then
exit;

sldb := TSQLiteDatabase.Create(slDBPath);
try

iID := sltb.FieldAsInteger(sltb.FieldIndex['ID']);


sltbU := sldb.GetUniTable('SELECT picture FROM testtable where ID = ' + inttostr(iID));
try

ms := sltbU.FieldAsBlob(sltbU.FieldIndex['picture']);

if (ms = nil) then
exit;

try

ms.Position := 0;

pic := TJPEGImage.Create;
pic.LoadFromStream(ms);

self.Image1.Picture.Graphic := pic;

pic.Free;

finally
ms.Free;
end;

finally
sltbU.Free;
end;

finally
sldb.Free;

end;


end;

procedure TForm1.FormCreate(Sender: TObject);
begin
self.sltb := nil;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
if sltb <> nil then  begin
      freeandnil(sltb);
end;
end;

procedure TForm1.btnLoadImageClick(Sender: TObject);
var
slDBpath: string;
sldb: TSQLiteDatabase;
iID: integer;
fs: TFileStream;

begin

slDBPath := ExtractFilepath(application.exename)
+ 'test.db';

if not FileExists(slDBPath) then begin
MessageDLg('Test.db does not exist. Click Test Sqlite 3 to create it.',mtInformation,[mbOK],0);
exit;
end;

if sltb = nil then
exit;

sldb := TSQLiteDatabase.Create(slDBPath);
try

if sltb.EOF then begin
MessageDLg('Table is at end of file.',mtInformation,[mbOK],0);
exit;
end;

iID := sltb.FieldAsInteger(sltb.FieldIndex['ID']);

//load an image
fs := TFileStream.Create(ExtractFileDir(application.ExeName) + '\sunset.jpg',fmOpenRead);
try

//insert the image into the db
sldb.UpdateBlob('UPDATE testtable set picture = ? WHERE ID = ' + inttostr(iID),fs);

finally
fs.Free;
end;

finally
sldb.Free;

end;

updateImage;

end;

procedure TForm1.btnBackClick(Sender: TObject);
begin
if sltb = nil then begin
MessageDLg('Table not initialised. Click Test Sqlite 3 to create it.',mtInformation,[mbOK],0);
exit;
end;

if not sltb.BOF then
begin
sltb.Previous;
updateFields;
end;

end;

procedure TForm1.btnBackupClick(Sender: TObject);
var
slDBpath: string;
sldb: TSQLiteDatabase;
sldbBak: TSQLiteDatabase;

begin

slDBPath := ExtractFilepath(application.exename);

if not FileExists(slDBPath + 'test.db') then begin
MessageDLg('Test.db does not exist. Click Test Sqlite 3 to create it.',mtInformation,[mbOK],0);
exit;
end;

sldb := TSQLiteDatabase.Create(slDBPath + 'test.db');
try
 sldbBak := TSQLiteDatabase.Create(slDBPath + 'testbak.db');
 try
 sldb.Backup(sldbBak);
 finally
 sldbBak.Free;
 end;

finally
sldb.Free;

end;


end;

procedure TForm1.btnForwardClick(Sender: TObject);
begin
if sltb = nil then begin
MessageDLg('Table not initialised. Click Test Sqlite 3 to create it.',mtInformation,[mbOK],0);
exit;
end;

if not slTb.IsLastRow then
begin
sltb.Next;
updateFields;
end;

end;

end.
